<?php

namespace Rbac\Components;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\Cache;

/**
 * Класс для работы с сервисом RBAC
 *
 * Class Rbac
 *
 * @package Rbac\Components
 */
class Rbac
{
    /** @var string $url URL сервиса */
    public $url = '';
    /** @var string $login Логин пользователя */
    public $login = '';
    /** @var string $project_name Символьный идентификатор проекта */
    public $projectName = '';
    /** @var string $password Пароль пользователя */
    public $password = '';
    /** @var string $authToken Токен авторизации */
    public $authToken = '';

    /**@var Client $client Клиент */
    public $client;

    /** @var string Тип запроса POST */
    public const TYPE_REQUEST_POST = 'POST';
    /** @var string Тип запроса GET */
    public const TYPE_REQUEST_GET = 'GET';

    /**
     * Localization constructor.
     *
     * @throws GuzzleException
     */
    public function __construct()
    {
        $this->client = new Client();
        $this->url = config('app.rbac.url');
        $this->login = config('app.rbac.login');
        $this->password = config('app.rbac.password');
        $this->projectName = config('app.rbac.projectName');
        $this->auth();
    }

    /**
     * Авторизация
     *
     * @throws GuzzleException
     */
    public function auth()
    {
        $data = [
            'login' => $this->login,
            'password' => $this->password,
        ];

        $key = $this->url . 'v1/auth' . json_encode($data);
        $result = Cache::get($key);
        if (empty($result)) {
            $result = $this->sendRequest(
                $this->url . '/v1/auth',
                $data,
                self::TYPE_REQUEST_POST,
                true
            );
            Cache::put($key, $result, 3600 * 24);
        }

        if (!empty($result['token'])) {
            $this->authToken = $result['token'];
        }
    }

    /**
     * Метод получения ролей с правами доступа
     *
     * @return array
     * @throws GuzzleException
     */
    public function getRoles(): array
    {
        return $this->sendRequest($this->url . '/v1/roles', [
            'project_name' => $this->projectName,
            'relations' => [
                'permissions',
                'users',
            ]
        ], self::TYPE_REQUEST_GET);
    }

    /**
     * Отправить запрос в сервис
     *
     * @param string $url URL запроса
     * @param array $data Данные запроса
     * @param string $typeRequest Тип запроса
     * @param bool $isAuth Является ли запрос авторизацией
     *
     * @return array|mixed
     * @throws GuzzleException
     */
    protected function sendRequest($url, $data, $typeRequest = self::TYPE_REQUEST_POST, $isAuth = false)
    {
        $headers = [];
        //Если запрос не на авторизацию, то надо вставить токен
        if (!$isAuth) {
            $headers = array_merge($headers, ['Authorization' => 'Bearer ' . $this->authToken]);
        }
        if ($typeRequest === self::TYPE_REQUEST_POST) {
            $options = [
                'form_params' => $data,
            ];
        } else {
            $options = [
                'query' => $data,
            ];
        }

        try {
            $result = $this->client
                ->request($typeRequest, $url, array_merge([
                    'headers' => $headers
                ], $options))
                ->getBody();
        } catch (\Exception $exception) {
            return [];
        }

        $result = $this->objectToArray(json_decode($result->getContents()));

        return $result['data'];
    }

    /**
     * Преобразование объекта в массив
     *
     * @param object $object Объект
     *
     * @return array|mixed
     */
    protected function objectToArray($object)
    {
        if (!is_object($object) && !is_array($object)) {
            return $object;
        }

        return array_map(self::class . '::objectToArray', (array)$object);
    }
}