<?php

namespace Rbac\Listeners;

use Rbac\Components\Rbac;
use Rbac\Events\UpdatePermissions as UpdatePermissionsEvent;
use GuzzleHttp\Exception\GuzzleException;
use Illuminate\Support\Facades\DB;
use Psr\Log\LoggerInterface;

/**
 * Обработчик события обновления прав доступа
 *
 * Class UpdatePermissions
 *
 * @package Rbac\Listeners
 */
class UpdatePermissions
{
    /**
     * Логгер
     *
     * @var LoggerInterface
     */
    private ?LoggerInterface $logger;

    /**
     * Handle the event.
     *
     * @param UpdatePermissionsEvent $event
     *
     * @return void
     * @throws GuzzleException
     * @throws \Exception
     */
    public function handle(UpdatePermissionsEvent $event)
    {
        $this->logger = $event->getLogger();
        $this->log('---Начало обновления прав доступа---');
        try {
            $roles = (new Rbac())->getRoles();
        } catch (\Exception $exception) {
            $this->log('Exception при получении ролей: ' . $exception->getMessage());
        }
        $this->log('Полученные роли', $roles);
        if (empty($roles)) {
            return;
        }
        if (!is_array($roles)) {
            $this->log('Ошибка!!! Роли не являются массивом: ' . print_r($roles, true));

            return;
        }
        DB::beginTransaction();
        try {
            $this->update($roles);
        } catch (\Exception $exception) {
            DB::rollBack();
            $this->log('Exception при обновлении: ' . $exception->getMessage());

            return;
        }
        DB::commit();
        $this->log('---Права доступа успешно обновлены---');
    }

    /**
     * Обновление ролей
     *
     * @param array $roles Роли
     */
    private function update($roles)
    {
        DB::table('rbac.roles_users')->truncate();
        DB::table('rbac.roles_permissions')->truncate();
        DB::table('rbac.roles')->truncate();
        DB::table('rbac.permissions')->truncate();
        foreach ($roles as $role) {
            DB::table('rbac.roles')->insert(['name' => $role['name']]);
            if (!empty($role['permissions'])) {
                foreach ($role['permissions'] as $permission) {
                    DB::table('rbac.permissions')->insert(['name' => $permission]);
                    DB::table('rbac.roles_permissions')->insert([
                        'role_name' => $role['name'],
                        'permission_name' => $permission
                    ]);
                }
            }
            if (!empty($role['users'])) {
                foreach ($role['users'] as $userId) {
                    DB::table('rbac.roles_users')->insert([
                        'role_name' => $role['name'],
                        'user_id' => $userId
                    ]);
                }
            }
        }
    }

    /**
     * Логирование
     *
     * @param string $message Сообщение
     * @param array $context Контекст
     */
    private function log($message, $context = [])
    {
        if (!empty($this->logger)) {
            $this->logger->info($message, $context);
        }
    }
}
